package eu.ztsh.wymiana.service;

import eu.ztsh.wymiana.exception.NoDataException;
import eu.ztsh.wymiana.model.Rates;
import lombok.RequiredArgsConstructor;
import org.assertj.core.util.VisibleForTesting;
import org.springframework.http.HttpStatusCode;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClient;

import java.time.Clock;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;

/**
 * NBP exchange rates service
 */
@Service
@RequiredArgsConstructor
public class NbpService {

    private final Clock clock;
    private final RestClient restClient;
    private static final String URI_PATTERN = "/api/exchangerates/rates/c/{code}/{date}/";
    private final DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    public double getSellRate(String currency) {
        return fetchData(currency, dtf.format(getFetchDate())).getRates().get(0).getAsk();
    }

    public double getBuyRate(String currency) {
        return fetchData(currency, dtf.format(getFetchDate())).getRates().get(0).getBid();
    }

    /**
     * Calculates date for data fetch.
     * <p>
     * Usually this would be today, but as rates are set only Mon-Fri, during weekends it is needed to fetch last friday rates.
     *
     * @return date for data fetch
     */
    @VisibleForTesting
    LocalDate getFetchDate() {
        var today = LocalDate.now(clock);
        return isWeekend(today) ? today.with(TemporalAdjusters.previous(DayOfWeek.FRIDAY)) : today;
    }

    @VisibleForTesting
    Rates fetchData(String code, String date) {
        return restClient.get().uri(URI_PATTERN, code.toLowerCase(), date)
            .retrieve()
            .onStatus(HttpStatusCode::is4xxClientError, ((request, response) -> {
                throw new NoDataException(code, date);
            }))
            .body(Rates.class);
    }

    private static boolean isWeekend(LocalDate today) {
        return today.getDayOfWeek() == DayOfWeek.SATURDAY
            || today.getDayOfWeek() == DayOfWeek.SUNDAY;
    }

    private record RatesCache(LocalDate date, double buy, double sell) {

    }

}
